pro read_genxcat, t0x, t1x, outdata, count=count, topdir=topdir, genfiles=genfiles, $
		initialize=initialize, sublevel=sublevel, debug=debug, fast=fast, $
	        dbfiles=dbfiles, catfiles=catfiles, deltat=deltat, status=status, loud=loud
;
;+   Name: rd_genxcat
;
;    Purpose: read a catalog which uses 'genx' format files
;
;    Input Parameters:
;      t0 - start time for search
;      t1 - stop time for search
;
;    Output Parameters:
;      outdata - output catalog data between t0 and t1
;d
;    Keyword Parameters:
;      TOPDIR - parent directories where catalogs reside
;      STATUS - success status 1=success=records between t0,t1
;      DELTAT - output - deltaT seconds between t0 and output
;      COUNT  - output - number of output records returned  
;         
;  
;    History:
;      9-Mar-1998 - S.L.Freeland - long planned 'gen' function
;                   finally implemented for TRACE  
;     23-Apr-1998 - dont clobber input (t0x,t1x)
;                   handle last file properly when no times are passed
;     15-Jun-1998 - S.L.Freeland - use another 'temporary' call
;     25-Jun-1998 - S.L.Freeland - protect against no data between t0,t1
;                   add DELTAT , STATUS, and COUNT keywords
;     19-Jul-1998 - S.L.Freeland - pre make array instead of concatenation
;      4-Aug-1998 - S.L.Freeland - copy_struct -> template (allow diff structs)
;  
;    Restrictions:
;       catalog files have GMT of first record embedded in file name
;
;    Calls:
;       file2time, time2file, restgen, 
;-
common read_genxcat, last_topdir, cfiles, cfids
debug=keyword_set(debug)
loud=keyword_set(loud)

initialize=keyword_set(initialize) or n_elements(cfiles) eq 0 or $
    n_elements(last_topdir) eq 0 

if initialize then begin
  if n_elements(sublevel) eq 0 then sublevel=0
  if data_chk(topdir,/string,/scalar) then begin
     if file_exist(topdir) then tdir=topdir else tdir=get_logenv(topdir)
  endif else tdir=curdir()
  for i=0,sublevel-1 do tdir=concat_dir(tdir,'*')
  catfiles=file_list(tdir,'*.genx',/cd)
  cfiles=catfiles
  cfids=extract_fid(cfiles)
  last_topdir=tdir
endif

fiddelim=(['.','_'])(strpos(cfids(0),'_') ne -1)

if n_elements(t0x) gt 0 then t0=t0x
if n_elements(t1x) gt 0 then t1=t1x

if n_elements(t0) gt 0 then begin
    fid0=time2file(timegrid(t0,day=-1,/string),delim=fiddelim)
    if n_elements(t1) eq 0 then t1=timegrid(t0,/day,/string)
    fid1=time2file(t1,delim=fiddelim)
endif else begin
   fid0=cfids(0)
   fid1=cfids(n_elements(cfids)-1)
endelse  

ss=where(cfids ge fid0 and cfids le fid1,sscnt)

if sscnt eq 0 then begin
  box_message,'No catalog entries satisfy your criteria...'
  return
endif  

if ss(0) gt 0 then ss=[ss(0)-1,ss]
sscnt=n_elements(ss)

logsizes=file_size(catfiles(ss))
maxsize=(where(logsizes eq max(logsizes)))(0)
restgen,file=catfiles(ss(maxsize)),structs
strsize=get_nbytes(structs(0),/str)                ; template structure

nout=round(total(logsizes)/strsize)                  ; worst case(?)
if debug then stop
outdata=replicate(structs(0),nout)

pointer=0
template=outdata(0)
for i=0, sscnt-1 do begin                     ; genx file loop
   if loud then box_message,'Reading:>> ' + catfiles(ss(i))
   restgen,file=catfiles(ss(i)),structs       ; restore one
   nstructs=n_elements(structs)               ; remember last number
   temp=replicate(template,nstructs)
   copy_struct,structs,temp
   delvarx,structs                            ; memory
   outdata(pointer)=temp                      ; insert into output
   pointer=pointer+nstructs                   ; pointer->
endfor

laststr=(pointer-1)<(n_elements(outdata)-1)

if debug then stop
outdata=temporary(outdata(0:laststr))                ; trim to valid contents

; trim times to actual if supplied
if n_elements(t1) gt 0 then begin
   ss=sel_timrange(outdata,t0,t1,/between)
   if ss(0) eq -1 then begin 
      ss=tim2dset(outdata,t0,delta=deltat)
      box_message,['No times between t0 and t1, returning record closest to t0', $
        'DeltaT (catrec,t0) = ' + strtrim(deltat,2) + ' seconds']
   endif
   outdata=outdata(ss)
endif

status=n_elements(deltat) eq 0

count=n_elements(outdata)
return
end
